"""
Containers Module - Checks for container-related metrics.
Used for detecting differences in containers across active/standby hosts.
"""
import subprocess
import json
import sys
from ..base import BaseExporter


class CustomExporter(BaseExporter):
    """
    Exporter for container-related checks.
    Runs checks related to container differences and reports metrics.
    """
    def metric_config(self):
        """Define metrics for container-related checks"""
        return {
            "dp_hactl_missing_containers_total": {"type": "Gauge", "labels": ["type", "problem"]},
        }

    def get_hostname(self):
        """Get the hostname of the machine running the script"""
        try:
            hostname_result = subprocess.run(
                ["hostname"],
                capture_output=True,
                text=True,
                timeout=5
            )
            return hostname_result.stdout.strip()
        except subprocess.TimeoutExpired:
            # Exit silently on timeout
            sys.exit(1)
        except subprocess.SubprocessError:
            # Fallback to a default value if hostname command fails
            return "unknown"

    def generate(self):
        """Run all container-related checks and collect metrics"""
        try:
            self._check_container_diff()
        except subprocess.TimeoutExpired:
            # Exit silently on timeout
            sys.exit(1)
        except subprocess.SubprocessError as e:
            # Preserve the original exit code from the command
            if hasattr(e, 'returncode'):
                sys.exit(e.returncode)
            else:
                sys.exit(1)
        except Exception:
            # For non-subprocess errors, still exit with a non-zero code
            sys.exit(1)

    def _check_container_diff(self):
        """Check for container differences between hosts"""
        missing_standby_metric = self.metrics.get("dp_hactl_missing_containers_total")
        
        # Run the command with check=True - let subprocess errors propagate
        # to generate() which will exit with the original error code
        result = subprocess.run(
            ["dp-ha-ctl", "container-diff"],
            capture_output=True,
            text=True,
            timeout=15,
            check=True  # Will raise CalledProcessError if command fails
        )
        
        # If we get here, the command executed successfully
        
        # Check if the output is empty
        if not result.stdout.strip():
            # No output means no missing containers, set counts to 0
            missing_standby_count = 0
            missing_primary_count = 0
            # Always use all defined labels, with "none" for problem in normal cases
            missing_standby_metric.labels(type="missing_standby", problem="none").set(missing_standby_count)
            missing_standby_metric.labels(type="missing_primary", problem="none").set(missing_primary_count)
        else:
            # Check if the output is valid JSON
            try:
                container_diff_output = json.loads(result.stdout)
                # Extract the counts, default to 0 if missing or empty
                missing_standby_count = len(container_diff_output.get("ContainerDiff", {}).get("missing_standby", []))
                missing_primary_count = len(container_diff_output.get("ContainerDiff", {}).get("missing_primary", []))
                # Always use all defined labels, with "none" for problem in normal cases
                missing_standby_metric.labels(type="missing_standby", problem="none").set(missing_standby_count)
                missing_standby_metric.labels(type="missing_primary", problem="none").set(missing_primary_count)
            except json.JSONDecodeError:
                # If output is not valid JSON but the command succeeded, treat it as no missing containers
                missing_standby_metric.labels(type="missing_standby", problem="none").set(0)
                missing_standby_metric.labels(type="missing_primary", problem="none").set(0)
