"""
Core Module - Basic dp-ha-ctl checks.
Used for metrics relating to core dp-ha-ctl functionality like Redis connection and service status.
"""
import subprocess
import json
import sys
from ..base import BaseExporter


class CustomExporter(BaseExporter):
    """
    Exporter for core dp-ha-ctl checks.
    Runs checks related to Redis connection and service status.
    """
    def metric_config(self):
        """Define metrics for core functionality checks"""
        return {
            "dp_hactl_redis_connected": {"type": "Gauge", "labels": []},
            "dp_hactl_service_up": {"type": "Gauge", "labels": []},
        }

    def get_hostname(self):
        """Get the hostname of the machine running the script"""
        try:
            hostname_result = subprocess.run(
                ["hostname"],
                capture_output=True,
                text=True,
                timeout=5
            )
            return hostname_result.stdout.strip()
        except subprocess.TimeoutExpired:
            # Exit silently on timeout
            sys.exit(1)
        except subprocess.SubprocessError:
            # Fallback to a default value if hostname command fails
            return "unknown"

    def generate(self):
        """Run all core checks and collect metrics"""
        try:
            self._check_redis_connection()
            self._check_service_status()
        except subprocess.TimeoutExpired:
            # Exit silently on timeout
            sys.exit(1)
        except subprocess.SubprocessError as e:
            # Preserve the original exit code from the command
            if hasattr(e, 'returncode'):
                sys.exit(e.returncode)
            else:
                sys.exit(1)
        except Exception:
            # For non-subprocess errors, still exit with a non-zero code
            sys.exit(1)

    def _check_redis_connection(self):
        """Check if Redis backend is connected"""
        redis_connected_metric = self.metrics.get("dp_hactl_redis_connected")
        
        # Run the command with check=True, allowing any errors to propagate up
        # This will exit the script with the original error code if dp-ha-ctl fails
        subprocess.run(
            ["dp-ha-ctl", "ha-status"],
            capture_output=True,
            text=True,
            timeout=15,
            check=True  # Will raise CalledProcessError if command fails
        )
        
        # If we get here, the command was successful (exit code 0)
        redis_connected_metric.set(1)

    def _check_service_status(self):
        """Check if dp-ha-ctl service is up"""
        service_up_metric = self.metrics.get("dp_hactl_service_up")
        
        # Run the command with check=True, allowing any errors to propagate up
        # This will exit the script with the original error code if dp-ha-ctl fails
        result = subprocess.run(
            ["dp-ha-ctl", "dump-all-state"],
            capture_output=True,
            text=True,
            timeout=15,
            check=True  # Will raise CalledProcessError if command fails
        )
        
        # If we get here, check if output is valid JSON
        try:
            json.loads(result.stdout)
            service_up_metric.set(1)
        except json.JSONDecodeError:
            # Still report metrics if JSON is invalid but command succeeded
            service_up_metric.set(0)
