"""
Logging Checks Module - Checks for log-related metrics.
Used for metrics relating to log entries and patterns.
"""
import os
import gzip
import datetime
import subprocess
import sys
from ..base import BaseExporter


class CustomExporter(BaseExporter):
    """
    Exporter for logging-related checks.
    Runs checks related to log files and reports metrics.
    """
    def metric_config(self):
        """Define metrics for logging-related checks"""
        return {
            "hamanager_redis_auth_failures": {"type": "Gauge", "labels": []},
            "hamanager_redis_timeouts": {"type": "Gauge", "labels": []},
        }

    def get_hostname(self):
        """Get the hostname of the machine running the script"""
        try:
            hostname_result = subprocess.run(
                ["hostname"],
                capture_output=True,
                text=True,
                timeout=5
            )
            return hostname_result.stdout.strip()
        except subprocess.TimeoutExpired:
            # Exit silently on timeout
            sys.exit(1)
        except subprocess.SubprocessError:
            # Fallback to a default value if hostname command fails
            return "unknown"

    def generate(self):
        """Run all logging-related checks and collect metrics"""
        try:
            self._check_redis_auth_failures()
            self._check_redis_timeouts()
        except subprocess.TimeoutExpired:
            # Exit silently on timeout
            sys.exit(1)
        except subprocess.SubprocessError as e:
            # Preserve the original exit code from the command
            if hasattr(e, 'returncode'):
                sys.exit(e.returncode)
            else:
                sys.exit(1)
        except Exception:
            # For non-subprocess errors, still exit with a non-zero code
            sys.exit(1)

    def _count_log_occurrences(self, search_pattern, time_window):
        """Count occurrences in logs within the last N minutes"""
        log_dir = "/var/log/hamanager/"
        
        # Get the timestamp N minutes ago
        start_time = datetime.datetime.now() - datetime.timedelta(minutes=time_window)
        threshold_timestamp = datetime.datetime.timestamp(start_time)
        
        # Initialize counter
        occurrences = 0
        
        # Get list of log files
        try:
            log_files = [f for f in os.listdir(log_dir) if f.startswith("hamanager.log")]
        except (FileNotFoundError, PermissionError):
            return 0
        
        # Loop over log files, including rotated ones
        for log_file in log_files:
            full_path = os.path.join(log_dir, log_file)
            
            # Skip files that haven't been modified within our time window
            try:
                mtime = os.path.getmtime(full_path)
                if mtime < threshold_timestamp:
                    # File is older than our time window, skip it
                    continue
            except (FileNotFoundError, PermissionError):
                continue
            
            # Check if the log file is compressed
            if log_file.endswith(".gz"):
                try:
                    with gzip.open(full_path, 'rt') as f:
                        for line in f:
                            # Extract first 19 chars of timestamp (YYYY-MM-DD HH:MM:SS) from log line
                            try:
                                timestamp_str = line[:19]
                                timestamp = datetime.datetime.strptime(timestamp_str, "%Y-%m-%d %H:%M:%S")
                                if timestamp >= start_time and search_pattern in line:
                                    occurrences += 1
                            except (ValueError, IndexError):
                                continue
                except (IOError, PermissionError):
                    continue
            else:
                try:
                    with open(full_path, 'r') as f:
                        for line in f:
                            # Extract first 19 chars of timestamp (YYYY-MM-DD HH:MM:SS) from log line
                            try:
                                timestamp_str = line[:19]
                                timestamp = datetime.datetime.strptime(timestamp_str, "%Y-%m-%d %H:%M:%S")
                                if timestamp >= start_time and search_pattern in line:
                                    occurrences += 1
                            except (ValueError, IndexError):
                                continue
                except (IOError, PermissionError):
                    continue
        
        return occurrences

    def _check_redis_auth_failures(self, time_window=5):
        """Check for Redis auth failures in logs"""
        auth_failures_metric = self.metrics.get("hamanager_redis_auth_failures")
        
        search_pattern = "invalid username-password pair"
        auth_failures = self._count_log_occurrences(search_pattern, time_window)
        
        auth_failures_metric.set(auth_failures)

    def _check_redis_timeouts(self, time_window=5):
        """Check for Redis timeouts in logs"""
        timeouts_metric = self.metrics.get("hamanager_redis_timeouts")
        
        search_pattern = "Redis exception:.*Timeout reading from socket"
        timeouts = self._count_log_occurrences(search_pattern, time_window)
        
        timeouts_metric.set(timeouts)
