"""
Systemctl Module - Checks for systemctl-based services.
Used for metrics relating to system service status.
"""
import subprocess
import sys
from ..base import BaseExporter


class CustomExporter(BaseExporter):
    """
    Exporter for systemctl-based checks.
    Runs checks related to system services and reports metrics.
    """
    def metric_config(self):
        """Define metrics for systemctl-based checks"""
        return {
            "podman_socket_active": {"type": "Gauge", "labels": []},
            "hamanager_service_active": {"type": "Gauge", "labels": []},
        }

    def get_hostname(self):
        """Get the hostname of the machine running the script"""
        try:
            hostname_result = subprocess.run(
                ["hostname"],
                capture_output=True,
                text=True,
                timeout=5
            )
            return hostname_result.stdout.strip()
        except subprocess.TimeoutExpired:
            # Exit silently on timeout
            sys.exit(1)
        except subprocess.SubprocessError:
            # Fallback to a default value if hostname command fails
            return "unknown"

    def generate(self):
        """Run all systemctl-based checks and collect metrics"""
        try:
            self._check_podman_socket()
            self._check_hamanager_service()
        except subprocess.TimeoutExpired:
            # Exit silently on timeout
            sys.exit(1)
        except subprocess.SubprocessError as e:
            # Preserve the original exit code from the command
            if hasattr(e, 'returncode'):
                sys.exit(e.returncode)
            else:
                sys.exit(1)
        except Exception:
            # For non-subprocess errors, still exit with a non-zero code
            sys.exit(1)

    def _check_podman_socket(self):
        """Check if podman.socket is active"""
        podman_socket_metric = self.metrics.get("podman_socket_active")
        
        # Use check=False since is-active returning non-zero just means inactive
        # But let any other subprocess errors propagate up
        result = subprocess.run(
            ["systemctl", "is-active", "podman.socket"],
            capture_output=True,
            text=True,
            timeout=5,
            check=False  # Non-zero exit code is expected for inactive services
        )
        
        # systemctl is-active returns 0 if active, non-zero if inactive
        podman_socket_active = 1 if result.returncode == 0 else 0
        podman_socket_metric.set(podman_socket_active)

    def _check_hamanager_service(self):
        """Check if hamanager.service is active"""
        hamanager_service_metric = self.metrics.get("hamanager_service_active")
        
        # Use check=False since is-active returning non-zero just means inactive
        # But let any other subprocess errors propagate up
        result = subprocess.run(
            ["systemctl", "is-active", "hamanager.service"],
            capture_output=True,
            text=True,
            timeout=5,
            check=False  # Non-zero exit code is expected for inactive services
        )
        
        # systemctl is-active returns 0 if active, non-zero if inactive
        hamanager_service_active = 1 if result.returncode == 0 else 0
        hamanager_service_metric.set(hamanager_service_active)
